@@ -30,7 +30,9 @@
  *   
 */
 #include <config.h>
+#ifndef WIN32
 #include <stdlib.h>
+#endif
 #include <stdio.h>
 #include <stdint.h>
 #include <unistd.h>
@@ -58,7 +60,7 @@ struct timeval tm_main_total;
 fsm_t *fsm = 0;
 /*-------------------------------------------------------------------------*/
 void usage() {
-  printf("ophcrack 3.6.1 by Objectif Securite (http://www.objectif-securite.ch)\n\n");
+  printf("ophcrack %s by Objectif Securite (http://www.objectif-securite.ch)\n\n", PACKAGE_VERSION);
   printf("Usage: ophcrack [OPTIONS]\n");
   printf("Cracks Windows passwords with Rainbow tables\n\n");
   printf("  -a              disable audit mode (default)\n");
@@ -341,7 +343,7 @@ void display_work(ophcrack_t *crack, msg_work_t *msg, char *info) {
 
   hash_t *hsh  = msg->hsh;
   table_t *tbl = msg->tbl;
-  int which = msg->which;
+  ophkind_t kind = msg->kind;
   int cmin  = msg->cmin;
   int cmax  = msg->cmax;
 
@@ -351,7 +353,7 @@ void display_work(ophcrack_t *crack, msg_work_t *msg, char *info) {
   
   fprintf(arg->lfile, "%s", info);
 
-  switch (which) {
+  switch (kind) {
   case find:
     fprintf(arg->lfile, "Find chain end for ");
     break;
@@ -405,28 +407,50 @@ void display_load(ophcrack_t *crack, msg_load_t *msg, char *info) {
   if (arg->quiet && arg->lfile == stdout) return;
 
   table_t *tbl = msg->tbl;
-  int which = msg->which;
+  table_preload_t preload = msg->preload;
   int done  = msg->done;
   uint64_t size = msg->size >> 20;
 
   char *name = tbl->name;
   int tidx   = tbl->idx;
-  
+
   fprintf(arg->lfile, "%s", info);
 
-  if (done)
-    fprintf(arg->lfile, "Done preloading ");
-  else
-    fprintf(arg->lfile, "Preloading ");
+  if (preload >= preload_init && preload <= preload_srt) {
+    if (done)
+      fprintf(arg->lfile, "Done preloading ");
+    else
+      fprintf(arg->lfile, "Preloading ");
+  }
 
-  if (which == 1)
+  switch (preload) {
+  case preload_init:
+    if (tbl->shared_init)
+      fprintf(arg->lfile, "shared init data ");
+    else
+      fprintf(arg->lfile, "init data ");
+    break;
+
+  case preload_idx:
     fprintf(arg->lfile, ".index file ");
-  else if (which == 2)
-    fprintf(arg->lfile, ".bin file ");
-  else if (which == 3)
-    fprintf(arg->lfile, ".start file ");
+    break;
 
-  fprintf(arg->lfile, "(%llu MB) of table %s,%d\n", (long long unsigned)size, name, tidx);
+  case preload_end:
+    fprintf(arg->lfile, ".bin file ");
+    break;
+
+  case preload_srt:
+    fprintf(arg->lfile, ".start file ");
+    break;
+
+  default:
+    break;
+  }
+
+  if (preload >= preload_init && preload <= preload_srt) {
+    fprintf(arg->lfile, "(%llu MB) ", (long long unsigned)size);
+    fprintf(arg->lfile, "of table %s,%d\n", name, tidx);
+  }
 }
 /*-------------------------------------------------------------------------*/
 void display_found(ophcrack_t *crack, msg_found_t *found, char *info) {
@@ -452,59 +476,56 @@ void display_found(ophcrack_t *crack, msg_found_t *found, char *info) {
   fprintf(arg->lfile, "%s", info);
 
   if (!arg->auditmode) {
-
     if (strcmp(pwd, "") == 0 && showempty)
       fprintf(arg->lfile, "Found empty password for ");
     else
       fprintf(arg->lfile, "Found password %s for ", pwd);
-    
+
     switch (hsh->kind) {
     case lm1:
-      fprintf(arg->lfile, "1st LM hash #%d", hid);
+      fprintf(arg->lfile, "1st LM hash #%d ", hid);
       break;
-      
+
     case lm2:
-      fprintf(arg->lfile, "2nd LM hash #%d", hid);
+      fprintf(arg->lfile, "2nd LM hash #%d ", hid);
       break;
-      
+
     case nt:
       if (strcmp(hinfo, "") == 0)
-	fprintf(arg->lfile, "NT hash #%d", hid);
-      else 
-	if (arg->hideuname)
-	  fprintf(arg->lfile, "user ***** (NT hash #%d)", hid);
-	else
-	  fprintf(arg->lfile, "user %s (NT hash #%d)", hinfo, hid);
+        fprintf(arg->lfile, "NT hash #%d ", hid);
+      else
+        if (arg->hideuname)
+          fprintf(arg->lfile, "user ***** (NT hash #%d) ", hid);
+        else
+          fprintf(arg->lfile, "user %s (NT hash #%d) ", hinfo, hid);
       break;
-      
+
     default:
       break;
     }
   } else {
-
     if (strcmp(pwd, "") == 0 && showempty)
       fprintf(arg->lfile, "Found empty password for ");
     else
       fprintf(arg->lfile, "Found password of length %d for ", hsh->length);
-    
+
     switch (hsh->kind) {
     case lm1:
-      fprintf(arg->lfile, "1st LM hash #%d", hid);
+      fprintf(arg->lfile, "1st LM hash #%d ", hid);
       break;
-      
+
     case lm2:
-      fprintf(arg->lfile, "2nd LM hash #%d", hid);
+      fprintf(arg->lfile, "2nd LM hash #%d ", hid);
       break;
-      
+
     case nt:
-      fprintf(arg->lfile, "NT hash #%d", hid);
+      fprintf(arg->lfile, "NT hash #%d ", hid);
       break;
-      
+
     default:
       break;
     }
-    fprintf(arg->lfile, " category: %s ", category_string(hsh->category));
-    
+    fprintf(arg->lfile, "category: %s ", category_string(hsh->category));
   }
   if (tbl != 0)
     fprintf(arg->lfile, "in table %s #%d at column %d.", tbl->name, tbl->idx, col);
@@ -525,17 +546,17 @@ void display_status(ophcrack_t *crack, char *info) {
   int npwds_found  = crack->npwds_found;
   pthread_mutex_unlock(mutex);
 
-  printf("%s", info);  
+  printf("%s", info);
 
   // Brute force.
 
   if (crack->arg->bforce) {
-    double curr  = fsm->bforce_curr;
-    double total = fsm->bforce_total;
+    uint64_t curr  = fsm->bforce_curr;
+    uint64_t total = fsm->bforce_total;
     int ratio = 0;
 
     if (total > 0)
-      ratio = (int)ceil(100*curr / total);
+      ratio = (100*curr) / total;
 
     if (ratio < 100)
       printf("brute force (%d%%); ", ratio);
@@ -544,17 +565,19 @@ void display_status(ophcrack_t *crack, char *info) {
   // Preload.
 
   if (fsm->state == st_preload) {
-    double curr  = fsm->psize_curr;
-    double total = fsm->psize_total;
+    uint64_t curr  = fsm->psize_curr;
+    uint64_t total = fsm->psize_total;
     int ratio = 0;
-    
+
     if (total > 0)
-      ratio = (int)ceil(100*curr / total);
-    
+      ratio = (100*curr) / total;
+
     if (ratio < 100)
       printf("preload (%d%%); ", ratio);
-    else
+    else {
+      assert(curr == total);
       printf("preload done; ");
+    }
   }
 
   // Work.
@@ -563,7 +586,7 @@ void display_status(ophcrack_t *crack, char *info) {
     list_t *tables = crack->tables;
     list_nd_t *nd;
 
-    int total_cmax = 0;
+    int mean_cmax = 0;
     int count = 0;
 
     for (nd = tables->head; nd != 0; nd = nd->next) {
@@ -571,19 +594,23 @@ void display_status(ophcrack_t *crack, char *info) {
 
       if (!tbl->enabled) continue;
 
-      int ncols = tbl->ncols;
-      int cmax  = tbl->cmax;
+      uint64_t ncols = tbl->ncols;
+      uint64_t cmax  = tbl->cmax;
 
-      total_cmax += (1000*(ncols-cmax)) / ncols;
+      uint64_t n = ncols * ncols;
+      uint64_t dcmax = ncols-cmax;
+      int ratio_cmax = (100*dcmax*dcmax) / n;
+
+      mean_cmax += ratio_cmax;
       ++count;
     }
 
     if (count > 0)
-      total_cmax /= 10*count;
+      mean_cmax /= count;
 
     int ndone = nenabled - nactive - nremain;
 
-    printf("search (%d%%); ", total_cmax);
+    printf("search (%d%%); ", mean_cmax);
     printf("tables: total %d, done %d, using %d; ", nenabled, ndone, nactive);
   }
 
@@ -902,7 +929,7 @@ int main(int argc, char **argv) {
   // application. The number of threads is generally set equal to the
   // number of cores in the CPU plus one.
 
-  ophcrack_t *crack = ophcrack_alloc(arg->nthreads, arg);
+  ophcrack_t *crack = ophcrack_alloc(arg->nthreads+1, arg);
   ophcrack_start(crack);
 
   // Load the hashes from the pwdump file if one has been provided.
