@@ -53,7 +53,6 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
   this->arg = crack->arg;
   this->fsm = fsm_alloc(crack);
 
-  samDialog = 0;
   tableDialog = 0;
   helpDialog = 0;
   exportDialog = 0;
@@ -95,9 +94,10 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   tableModel = new TableModel(tableView);
   tableView->setModel(tableModel);
+  tableView->hideColumn(TableModel::DIR_COL_IDX);
 
   progDelegate = new ProgDelegate(tableView);
-  tableView->setItemDelegateForColumn(3, progDelegate);
+  tableView->setItemDelegateForColumn(TableModel::PROGRESS_COL_IDX, progDelegate);
 
   connect(tableModel, SIGNAL(itemChanged(QStandardItem*)),
 	  this, SLOT(tableItemChanged(QStandardItem*)));
@@ -152,8 +152,6 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
   samFile    = loadMenu->addAction("Encrypted SAM");
 #ifdef WIN32
   localSam2  = loadMenu->addAction("Local SAM with samdump2");
-  localSam   = loadMenu->addAction("Local SAM with pwdump6");
-  remoteSam  = loadMenu->addAction("Remote SAM");
 #endif
 
   connect(singleHash, SIGNAL(triggered()), this, SLOT(loadSingleHash()));
@@ -162,8 +160,6 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
   connect(sessFile, SIGNAL(triggered()), this, SLOT(loadHashFile()));
 #ifdef WIN32
   connect(localSam2, SIGNAL(triggered()), this, SLOT(loadLocalSamSamdump()));
-  connect(localSam, SIGNAL(triggered()), this, SLOT(loadLocalSamPwdump()));
-  connect(remoteSam, SIGNAL(triggered()), this, SLOT(loadRemoteSam()));
 #endif
 
   loadButton->setMenu(loadMenu);
@@ -612,7 +608,7 @@ void OphcrackGUI::loadLocalSamSamdump(void) {
     buff = QString("Error in samdump2: get_live_syskey function");
   
   else if (npwds == -2) 
-    buff = QString("Error in samdump2: get_sam function\n\nMake sure ophcrack is running with administrator rights to perform this action. On Windows Vista/7, right-click on ophcrack icon and select run as administrator.");
+    buff = QString("Error in samdump2: get_sam function\n\nMake sure ophcrack is running with administrator rights to perform this action. On Windows Vista and later, right-click on ophcrack icon and select run as administrator.\n\nThis function is not supported on Windows 10 Anniversary update. Use mimikatz (https://github.com/gentilkiwi/mimikatz/releases/) to dump local hashes: mimikatz privilege::debug  token::elevate lsadump::sam");
     
   else if (npwds == -3)
     buff = QString("Error in samdump2: samdump2 function");
@@ -639,104 +635,6 @@ void OphcrackGUI::loadLocalSamSamdump(void) {
   list_free(hashes);    
 }
 //---------------------------------------------------------------------------
-void OphcrackGUI::loadLocalSamPwdump(void) {  
-  // Retrieve the hashes from the local SAM with pwdump6.
-  
-  list_t *hashes = list_alloc();
-  list_nd_t *nd;
-
-  int maxhid = crack->maxhid;
-  int ret = exec_local_pwdump6(hashes, maxhid);
-
-  // If some hashes have been found, then we add them to the current
-  // list of hashes.
-
-  if (ret > 0) {
-    for (nd = hashes->head; nd != 0; nd = nd->next) {
-      hash_t *hsh = (hash_t*)nd->data;
-      
-      ophcrack_add_hash(crack, hsh);
-      hashModel->insertHash(hsh);
-    }
-
-    hashesAdded(hashes);
-  } 
-
-  // Otherwise, we notify the user that a problem occured.
-
-  else {
-    QMessageBox msgBox(QMessageBox::Warning,
-		       "Warning",
-		       QString("Cannot dump local SAM with pwdump6."),
-		       QMessageBox::Ok);
-    msgBox.exec();
-  }
-    
-  list_free(hashes);  
-}
-//---------------------------------------------------------------------------
-void OphcrackGUI::loadRemoteSam(void) {
-  if (samDialog == 0)
-    samDialog = new SamDialog(this);
-  
-  if (samDialog->exec() == QDialog::Accepted) {
-    char *host  = strdup(samDialog->getHost());
-    char *share = strdup(samDialog->getShare());
-    char *user  = strdup(samDialog->getUser());
-
-    // If the host and share are not empty, then we can proceed.
-
-    if (host[0] != 0 || share[0] != 0) {
-      list_t *hashes = list_alloc();
-      list_nd_t *nd;
-      
-      int maxhid = crack->maxhid;
-      int ret = exec_remote_pwdump6(hashes, host, share, user, maxhid);
-      
-      // If some hashes have been found, then we add them to the current
-      // list of hashes.
-      
-      if (ret > 0) {
-	for (nd = hashes->head; nd != 0; nd = nd->next) {
-	  hash_t *hsh = (hash_t*)nd->data;
-	  
-	  ophcrack_add_hash(crack, hsh);
-	  hashModel->insertHash(hsh);
-	}
-	
-	fsm_reset_bforce(fsm);
-	fsm_reset_preload(fsm);
-      }
-
-      // Otherwise, we notify the user that a problem occured.
-      
-      else {
-	QMessageBox msgBox(QMessageBox::Warning,
-			   "Warning",
-			   QString("Cannot dump remote SAM with pwdump6."),
-			   QMessageBox::Ok);
-	msgBox.exec();
-      }
-
-      list_free(hashes);
-    }
-
-    // Warn the user that the host and/or the share are empty.
-
-    else {
-      QMessageBox msgBox(QMessageBox::Warning,
-			 "Warning",
-			 QString("You must specify a host and a share."),
-			 QMessageBox::Ok);
-      msgBox.exec();
-    }
-    
-    free(host);
-    free(share);
-    free(user);
-  }
-}
-//---------------------------------------------------------------------------
 void OphcrackGUI::save(void) {
   QString fileName;
 
@@ -895,15 +793,20 @@ void OphcrackGUI::handleMessages(void) {
       table_t *tbl = load->tbl;
 
       if (tbl && load->done)
-	tableModel->updateTable(tbl);
+        tableModel->updateTable(tbl);
+
+      else if (tbl == 0 && load->size == 0) {
+        for (list_nd_t *nd = crack->enabled->head; nd != 0; nd = nd->next)
+          tableModel->updateTable((table_t*)nd->data);
+      }
     }
-    
+
     // A table has been unloaded.
-    
+
     else if (msg->kind == msg_unload) {
       msg_load_t *load = (msg_load_t*)msg->data;
       table_t *tbl = load->tbl;
-      
+
       if (tbl)
 	tableModel->updateTable(tbl);
     }
@@ -1170,12 +1073,12 @@ void OphcrackGUI::updateStatus(void) {
 
   // Preload progress.
 
-  double psize_curr  = fsm->psize_curr;
-  double psize_total = fsm->psize_total;
+  uint64_t psize_curr  = fsm->psize_curr;
+  uint64_t psize_total = fsm->psize_total;
   int psize_ratio = 0;
 
   if (psize_total > 0)
-    psize_ratio = (int)ceil(100*psize_curr / psize_total);
+    psize_ratio = (100*psize_curr) / psize_total;
 
   QString preloadStr = "waiting";
 
@@ -1184,7 +1087,7 @@ void OphcrackGUI::updateStatus(void) {
 
   else if (psize_total > 0)
     preloadStr = QString("%1%").arg(psize_ratio);
-      
+
   preloadEdit->setText(preloadStr);
 
   // Brute force.
@@ -1193,12 +1096,12 @@ void OphcrackGUI::updateStatus(void) {
     bforceEdit->setText("disabled");
 
   else {
-    double bforce_curr  = fsm->bforce_curr;
-    double bforce_total = fsm->bforce_total;
+    uint64_t bforce_curr  = fsm->bforce_curr;
+    uint64_t bforce_total = fsm->bforce_total;
     int bforce_ratio = 0;
 
     if (bforce_total > 0)
-      bforce_ratio = (int)ceil(100*bforce_curr / bforce_total);
+      bforce_ratio = (100*bforce_curr) / bforce_total;
 
     QString bforceStr = "waiting";
 
@@ -1289,10 +1192,10 @@ void OphcrackGUI::updateConfig(void) {
 }
 //---------------------------------------------------------------------------
 void OphcrackGUI::threadChanged(int x) {
-  arg->nthreads = x;
-
-  if (x != crack->sched->nthreads)
+  if (x != crack->arg->nthreads) {
+    arg->nthreads = x;
     threadLabel->setText(QString("%1 (restart required)").arg(x));
+  }
   else
     threadLabel->setText(QString("%1").arg(x));
 }
