@@ -41,7 +41,6 @@
 
 #include "ui_ophcrackgui.h"
 #include "tabledialog.h"
-#include "samdialog.h"
 #include "hashmodel.h"
 #include "tablemodel.h"
 #include "progdelegate.h"
@@ -71,9 +70,7 @@ private slots:
   void loadSingleHash(void);
   void loadHashFile(void);
   void loadSamFile(void);
-  void loadLocalSamPwdump(void);
   void loadLocalSamSamdump(void);
-  void loadRemoteSam(void);
   void save(void);
   void exportCSV(void);
   void delSelection(void);
@@ -118,7 +115,6 @@ private:
 
   ophcrack_t *crack;
   HashModel *hashModel;
-  SamDialog *samDialog;
   TableModel *tableModel;
   TableDialog *tableDialog;
   ProgDelegate *progDelegate;
@@ -135,8 +131,6 @@ private:
   QAction *samFile;
   QAction *sessFile;
   QAction *localSam2;
-  QAction *localSam;
-  QAction *remoteSam;
 
   QMenu *saveMenu;
   QAction *saveFile;
